@@ -15,7 +15,9 @@ from mo_logs import logger
 from mo_json import JxType, JX_TEXT
 from mo_streams._utils import chunk_bytes, Stream
 
-ObjectStream, StringStream, File_usingStream, Typer = expect("ObjectStream", "StringStream", "File_usingStream", "Typer")
+ObjectStream, StringStream, File_usingStream, Typer = expect(
+    "ObjectStream", "StringStream", "File_usingStream", "Typer"
+)
 
 
 DEBUG = True
@@ -38,14 +40,17 @@ class ByteStream(Stream):
         def read():
             # ZIP HAS DIRACTORY AT END OF FILE, MUST READ WHOLE THING
             reader = self.reader
-            if not reader.seekable():
-                reader = BytesIO(b"".join(chunk_bytes(reader)))
-            with ZipFile(reader, mode="r") as archive:
-                for info in archive.filelist:
-                    yield File_usingStream(
-                        info.filename,
-                        lambda: ByteStream(archive.open(info.filename, "r")),
-                    ), {"name": info.filename}
+            try:
+                if not reader.seekable():
+                    reader = BytesIO(b"".join(chunk_bytes(reader)))
+                with ZipFile(reader, mode="r") as archive:
+                    for info in archive.filelist:
+                        yield File_usingStream(
+                            info.filename,
+                            lambda: ByteStream(archive.open(info.filename, "r")),
+                        ), {"name": info.filename}
+            finally:
+                reader.close()
 
         return ObjectStream(read(), Typer(type_=File_usingStream), JxType(name=JX_TEXT))
 
@@ -56,9 +61,7 @@ class ByteStream(Stream):
         """
         from zstandard import ZstdDecompressor
 
-        stream_reader = (
-            ZstdDecompressor(max_window_size=2147483648).stream_reader(self.reader)
-        )
+        stream_reader = ZstdDecompressor().stream_reader(self.reader, closefd=True)
         return ByteStream(stream_reader)
 
     def from_tar(self):
@@ -80,11 +83,14 @@ class ByteStream(Stream):
                 )
 
         def read():
-            while True:
-                info = tf.next()
-                if not info:
-                    return
-                yield file(info), {"name": info.name}
+            try:
+                while True:
+                    info = tf.next()
+                    if not info:
+                        return
+                    yield file(info), {"name": info.name}
+            finally:
+                self.reader.close()
 
         return ObjectStream(read(), Typer(type_=File_usingStream), JxType(name=JX_TEXT))
 
@@ -126,4 +132,5 @@ class ByteStream(Stream):
             if self.verbose:
                 logger.warn("problem with s3 upload", cause=cause)
 
+
 export("mo_streams._utils", ByteStream)
